@@ -1,9 +1,9 @@
 package Example::Schema::Result::Person;
 
+use Example::Base;
 use base 'Example::Schema::Result';
 
 __PACKAGE__->table("person");
-__PACKAGE__->load_components(qw/EncodedColumn/);
 
 __PACKAGE__->add_columns(
   id => { data_type => 'bigint', is_nullable => 0, is_auto_increment => 1 },
@@ -14,10 +14,7 @@ __PACKAGE__->add_columns(
     data_type => 'varchar',
     is_nullable => 0,
     size => 64,
-    #    encode_column => 1,
-    #    encode_class  => 'Digest',
-    #    encode_args   => { algorithm => 'MD5', format => 'base64' },
-    #    encode_check_method => 'check_password',
+    bcrypt => 1,
   },
 );
 
@@ -32,19 +29,17 @@ __PACKAGE__->validates(first_name => (presence=>1, length=>[2,24]));
 __PACKAGE__->validates(last_name => (presence=>1, length=>[2,48]));
 
 __PACKAGE__->validates(credit_cards => (result_set=>+{validations=>1, min=>2, max=>4}, on=>'profile' ));
-#__PACKAGE__->validates(roles => (presence=>1, result_set=>+{validations=>1, min=>1}, on=>'profile' ));
-__PACKAGE__->validates(person_roles => (result_set=>+{validations=>1, min=>1}, on=>'profile' ));
+__PACKAGE__->accept_nested_for('credit_cards');
 
+__PACKAGE__->validates(person_roles => (result_set=>+{validations=>1, min=>1}, on=>'profile' ));
+__PACKAGE__->accept_nested_for('person_roles');
 
 __PACKAGE__->validates(profile => (result=>+{validations=>1}, allow_blank=>1 ));
-__PACKAGE__->accept_nested_for('person_roles');
+__PACKAGE__->accept_nested_for('profile');
 
 __PACKAGE__->set_primary_key("id");
 __PACKAGE__->add_unique_constraint(['username']);
 
-__PACKAGE__->accept_nested_for('profile');
-__PACKAGE__->accept_nested_for('credit_cards');
-
 __PACKAGE__->might_have(
   profile =>
   'Example::Schema::Result::Profile',
@@ -67,6 +62,7 @@ __PACKAGE__->many_to_many('roles' => 'person_roles', 'role');
 
 __PACKAGE__->accept_nested_for('roles');
 
+# TODO: I think these can be removed
 sub registered {
   my $self = shift;
   return $self->validated && $self->valid;
