@@ -10,6 +10,7 @@ use Valiant::Util 'debug';
 use Scalar::Util 'blessed';
 use Carp;
 use namespace::autoclean -also => ['debug'];
+use DBIx::Class::Valiant::Util::Exception::TooManyRows;
 
 with 'DBIx::Class::Valiant::Validates';
 with 'Valiant::Filterable';
@@ -20,6 +21,8 @@ export_methods ['filters', 'validates', 'filters_with', 'validates_with', 'accep
 __PACKAGE__->mk_classdata( _m2m_metadata => {} );
 __PACKAGE__->mk_classdata( auto_validation => 1 );
 __PACKAGE__->mk_classdata( _nested => [] );
+__PACKAGE__->mk_classdata( _src_info => '' );
+
 
 sub many_to_many {
   my $class = shift;
@@ -72,7 +75,8 @@ sub new { # also support for the filter role
     }
   }
   my %filtered = (%$attrs, %{$class->_process_filters(\%columns)});
-  return $class->next::method(\%filtered);
+  my $self = $class->next::method(\%filtered);
+  return $self;
 }
 
 sub accept_nested_for {
@@ -120,7 +124,11 @@ sub insert {
     debug 2, "Skipping insert for @{[$self]} because its invalid";
     return $self;
   }
-  ## delete $self->{__VALIANT_CREATE_ARGS};  We might need this at some point
+
+  if($self->{__valiant_donot_insert}) {
+    debug 2, "Skipping insert for @{[$self]} because its probably and _add";
+    return $self;
+  }
   return $self->next::method(@args);
 }
 
@@ -141,7 +149,7 @@ sub update {
   # Remove any relationed keys we didn't find with the allows nested
   my @rel_names = $self->result_source->relationships();
   my @m2m_names = keys  %{ $self->result_class->_m2m_metadata ||+{} };
-  debug 1, "Found related for @{[ $self ]} of @{[ join ',', @rel_names ]}";
+  debug 1, "Found related for @{[ ref $self ]} of @{[ join ',', @rel_names ]}";
 
   my %found = map { $_ => delete($upd->{$_})  } @rel_names, @m2m_names; # backcompat with old perl
 
@@ -164,7 +172,15 @@ sub update {
         $limit_proto->($self) :
         $limit_proto;
       my $num = scalar @{$related{$related}};
-      confess "Relationship $related can't create more than $limit rows at once" if $num > $limit;      
+
+      DBIx::Class::Valiant::Util::Exception::TooManyRows
+        ->throw(
+          limit=>$limit,
+          attempted=>$num,
+          related=>$related,
+          me=>$self->result_source->name,
+        ) if $num > $limit;
+     
     }
     debug 2, "Setting related '$related' for @{[ ref $self ]} ";
 
@@ -220,13 +236,10 @@ sub register_column {
 # namespace
 
 sub namespace {
-  my $self = shift;  
-  return '' unless ref $self;
-
+  my $self = shift;
   my $class = ref($self) ? ref($self) : $self; 
-  my $source_name = $self->result_source->source_name;
-  $class =~s/::${source_name}$//;
-  return $class;
+  my ($ns) = ($class =~m/^(.+)::.+$/);
+  return $ns;
 }
 
 # Trouble here is you can only inject one attribute per model.  Will be an
@@ -322,7 +335,7 @@ sub delete_if_in_storage {
 
 sub build_related {
   my ($self, $related, $attrs) = @_;
-  debug 2, "Building related entity '$related' for @{[ $self->model_name->human ]}";
+  debug 2, "Building related entity '$related' for @{[ ref $self ]}";
 
   my $related_obj = $attrs ? $self->find_or_new_related($related, $attrs) : $self->new_related($related, +{});
   return if $related_obj->in_storage;  #I think we can skip if its found
@@ -400,6 +413,11 @@ sub set_from_params_recursively {
           $self->mark_for_deletion;
         }
       }
+    } elsif( $param eq '_nop') {
+        if($params{$param}) {
+          debug 3, "Found _nop";
+          delete $params{$param};
+        }
     } elsif($param eq '_restore' && $params{$param}) {
       if($self->in_storage) {
         debug 3, "Unmarking record @{[ ref $self ]}, id @{[ $self->id ]} for deletion";
@@ -411,6 +429,7 @@ sub set_from_params_recursively {
     } elsif($param eq '_action') {
       my $action = $params{$param};
       $action = ref($action)||'' ? $action->[-1] : $action; # If action is a ref always use the last one
+      debug 3, "Found _action param with value $action";
       if($action eq 'delete') {
         if($self->in_storage) {
           debug 2, "Marking record @{[ ref $self ]}, id @{[ $self->id ]} for deletion";
@@ -526,6 +545,12 @@ sub set_multi_related_from_params {
   my @search_sets = ('primary');
   my %nested = $self->result_class->accept_nested_for;
 
+  # Gather the list of fields which are single type rels
+  my @single_rels =  grep {
+    my $rel_data = $self->$related->result_source->relationship_info($_);
+    ($rel_data && ($rel_data->{attrs}{accessor} eq 'single')) ? 1:0;
+  }  $self->$related->result_source->relationships;
+
   # Generally we alow finding the row via the PK only but the user can allow finding
   # by any unique if that's what they really want.
   if($nested{$related}{find_with_uniques}) {
@@ -537,7 +562,11 @@ sub set_multi_related_from_params {
   debug 2, "starting loop to update/create related $related (total @{[ scalar @param_rows ]} rows in loop)";
   foreach my $param_row (@param_rows) {
     debug 3, "top of new loop on param_rows";
-    delete $param_row->{_add};
+    if($param_row->{_nop}) {
+      debug 3, "Is a NOP row, so skipping";
+      next;
+    }
+    my $was_add = delete $param_row->{_add};
     my $related_model;
     if(blessed $param_row) {
       debug 1, "params are an object";
@@ -553,6 +582,7 @@ sub set_multi_related_from_params {
         my @key_columns = @{$uniques{$key}};
         my %matching = map { $_ => $param_row->{$_} } grep { exists $param_row->{$_} } @key_columns;
 
+
         next unless scalar(@key_columns) == scalar(keys(%matching)); # only match when its a full key match
         debug 2, "key $key exists in params";
 
@@ -565,33 +595,34 @@ sub set_multi_related_from_params {
             $related_model = $row;
             if($related_model) {
               debug 2, "found related model for $related with $key in cache";
-              %keys_used_to_find = %matching;
+              %keys_used_to_find = (%keys_used_to_find, %matching);
               last SEARCH_UNIQUE_KEYS
             }
           }
         }
 
-        # If it doesn't match in the cache then we have to find it in the DB
+        # If it doesn't match in the cache then we have to find it in the DB. TODO I'm skipping this
+        # lookup for now since I don't think we need it unless the user failed to properly prefetch.
         unless($related_model) {
           debug 2, "Trying to find $related in DB with key $key";
-          $related_model = $self->find_related($related, \%matching, +{key=>$key});
+          #$related_model = $self->find_related($related, \%matching, +{key=>$key});
           if($related_model) {
             debug 2, "found related model for $related with $key in DB";
-            %keys_used_to_find = %matching;
+            %keys_used_to_find = (%keys_used_to_find, %matching);
             last SEARCH_UNIQUE_KEYS
           }
         }
       }
 
-      # Ok so if we get here and there's no $related model that means the user did not give us
-      # enough info in $param_rows to find it either in the cache or in the DB (they didn't give us
-      # any uniques or PKs. But its now impossible they gave us enough info to find the row anyway.
-      # Sometimes you can find it if they gave you a relationship that was identifying or some combo
-      # of a relationship and when $self is in storage and has a PK that is FK to $related and part of
-      # a unique key.  Mostly we see this in m2m bridge style relationships.  For now we have this hack
+      # If we get here its possible there's a single type relation we can use to complete 
+      # the lookup.  TODO: Not sure if this should be a param setting in the nested_for data 
+      # (it probably should be).  Also not sure if we should limit the ->find_related to 
+      # just the FK rel.
 
-      debug 2, "Falling back to default find_related for $related using full param_row";
-      $related_model = $self->find_related($related, $param_row) unless $related_model || !%{$param_row};
+      if(grep {$param_row->{$_}} @single_rels) {
+        debug 2, "Falling back to default find_related for $related using full param_row";
+        $related_model = $self->find_related($related, $param_row) unless $related_model || !%{$param_row};
+      }
 
       # OK so we didnt find it via keys either in the cache or in the DB so that means
       # we need to just create it.
@@ -600,6 +631,12 @@ sub set_multi_related_from_params {
         $related_model = $self->new_related($related, +{});
       }
       debug 2, "About to set_from_params_recursively for @{[ ref $related_model ]}";
+
+      if($was_add) {
+        debug 3, "Since @{[ ref $related_model]} was an _add we won't validate since its empty";
+        $related_model->skip_validate;
+        $related_model->{__valiant_donot_insert} = 1;
+      }
       
       # Don't set params for the found keys (waste of time, no change)
       my %params_for_recursive = %$param_row;
@@ -607,8 +644,9 @@ sub set_multi_related_from_params {
       $related_model->set_from_params_recursively(%params_for_recursive);
 
       # Ok, so after set_from_params_recursively if the $related_model is not in storage
-      # we 'might' have enough info to actually load it.
-      if(!$related_model->in_storage) {
+      # we 'might' have enough info to actually load it now.
+      if(!$related_model->in_storage && !$related_model->is_marked_for_deletion) {
+        # TODO: We should skip this if we've already tried a full PK
         my $copy = $related_model->get_from_storage(); # If we now have the full PK we can find it
         $related_model = $copy if $copy;
       }
@@ -663,12 +701,6 @@ sub set_multi_related_from_params {
       $cb->($current);
     }
 
-    ## TODO to solve the 'is in a deleted branch' issue either when we mark for deletion
-    # we immediately recursively look into its related caches and mark all children as 'in a deleted branch
-    # OR we have the code 'is_in_deleted_branch' follow up all belongs to rels looking for 'marked_for_deletion
-    # OR we have some sort of index/map (not sure how tod this)
-    # Option one at first look seems the least painful / most performant
-
     push @related_models, $current if $current->in_storage; # don't preserve unsaved previous
   }
 
@@ -681,6 +713,7 @@ sub set_multi_related_from_params {
 
 sub set_single_related_from_params {
   my ($self, $related, $params) = @_;
+  my %nested = $self->result_class->accept_nested_for;
 
   # Is there an existing related object in the cache?  If so then we
   # will merge params with existing rather than create a new one or
@@ -798,8 +831,25 @@ sub set_single_related_from_params {
           #   $self->set_from_related($related, $related_result);
           $related_result->set_from_params_recursively(%$params);
         } else {
-          debug 3, "finding with params matching";
-          $related_result = $self->result_source->related_source($related)->resultset->find($params);  # TODO problably shoulld search on unique keys only
+          debug 3, "No PK for $related, lets see if there's other ways to find it";
+          if($nested{$related}{find_with_uniques}) {
+            debug 3, "finding $related for $self with find_with_uniques matching";
+            my %uniques = $self->result_source->related_source($related)->unique_constraints;
+            foreach my $unique (keys %uniques) {
+              next if $unique eq 'primary'; # already done
+              my @key_columns = @{$uniques{$unique}};
+              my %matching = map { $_ => $params->{$_} } grep { exists $params->{$_} } @key_columns;
+
+              next unless scalar(@key_columns) == scalar(keys(%matching)); # only match when its a full key match
+              debug 2, "key $unique exists in params";
+              $related_result = $self->result_source->related_source($related)->resultset->find(\%matching);
+              if($related_result) {
+                debug 3, "Found result via unique keys";
+                last;  
+              }
+            }
+          }
+
           unless($related_result) {
             debug 3, "Did not find result for $related so creating new result";
             $related_result = $self->new_related($related, $params);
@@ -1029,6 +1079,11 @@ For C<has_one> or C<might_have> relationships will force update the existing nes
 any exists) even if you fail to set the primary key.  Otherwise the current record will be
 deleted and a new one inserted.  Default is false.
 
+=item find_with_uniques
+
+Generally we only try to find a matching row in the DB if a primary key is given.   If you set 
+this to true then we also try to match using and unique keys establish for the related row.
+
 =head1 METHODS
 
 This component adds the following object methods. Please note
