@@ -239,8 +239,10 @@ sub add {
 
   if(my $exception = $options->{strict}) {
     my $message = $error->full_message;
-    throw_exception('Strict' => (msg=>$message)) if $exception == 1;
-    $exception->throw($message); # If not 1 then assume its a package name.
+    throw_exception('Strict' => (msg=>$message)) if $exception =~m/^\d+$/ && $exception == 1;
+    throw_exception('Strict' => (msg=>$exception)) if( (ref(\$exception)||'') eq 'SCALAR');
+    $exception->($self->object, $message) if( (ref($exception)||'') eq 'CODE');
+    $exception->throw($message); # If not 1 then assume its a package name or exception object.
   }
  
   $self->errors->push($error);
